; Group.e, by Larry Margolis
;
; Defines a SaveGroup command which saves the contents of the edit ring
; as a group, and a LoadGroup command which reloads that group, positioning
; each file as it was when the SaveGroup was executed.  For OS/2 2.0 users,
; optionally creates a desktop icon for the group.  EPM 6.0 can do this
; directly; users of other versions must extract the command file at the
; end and save it as a MAKEGRP.CMD in the PATH.

compile if not defined(SMALL)  -- If SMALL not defined, then being separately compiled
 include 'stdconst.e'            -- (needed for MB_ constants)
 define INCLUDING_FILE = 'GROUP.E'
const
   tryinclude 'MYCNF.E'        -- The user's configuration customizations.

 compile if not defined(SITE_CONFIG)
    const SITE_CONFIG = 'SITECNF.E'
 compile endif
 compile if SITE_CONFIG
    tryinclude SITE_CONFIG
 compile endif

const
 compile if not defined(NLS_LANGUAGE)
   NLS_LANGUAGE = 'ENGLISH'
 compile endif
include NLS_LANGUAGE'.e'          -- Needed for UNNAMED_FILE_NAME
compile endif  -- not defined(SMALL)

const
compile if not defined(INCLUDE_DESKTOP_SUPPORT)
   INCLUDE_DESKTOP_SUPPORT = 1
compile endif
compile if EVERSION >= 6
   CO_FAILIFEXISTS    = 0
   CO_REPLACEIFEXISTS = 1
   CO_UPDATEIFEXISTS  = 2
compile endif

defc savegroup =
   universal app_hini
   getfileid startfid
   do i=1 to filesinring(1)  -- Provide an upper limit; prevent looping forever
      if .filename=UNNAMED_FILE_NAME then
         if .last<>1 or textline(1)<>'' then
            activatefile startfid
            sayerror 'An unnamed file exists in the ring; it must have a name to save the ring.'
            return
         endif
      endif
      next_file
      getfileid curfile
      if curfile = startfid then leave; endif
   enddo  -- Loop through all files in ring

   group_name = arg(1)
   if group_name='' then
      group_name = entrybox('Group name')
   endif
   if group_name='' then
      return
   endif
   tempstr = queryprofile( app_hini,  group_name, 'ENTRIES')
   if tempstr<>'' then
      if MBID_OK <> winmessagebox('Save Group', 'Group already exists.  OK to replace it?', 16417) then  -- MB_OKCANCEL + MB_ICONEXCLAMATION + MB_MOVEABLE
         return
      endif
   endif

   do i=1 to filesinring(1)  -- Provide an upper limit; prevent looping forever
      call setprofile(app_hini, group_name, 'FILE'i, .filename)
      call setprofile(app_hini, group_name, 'POSN'i, .line .col .cursorx .cursory)
      next_file
      getfileid curfile
      if curfile = startfid then leave; endif
   enddo  -- Loop through all files in ring
   call setprofile(app_hini, group_name, 'ENTRIES', i)

   if tempstr<>'' & tempstr>i then
      do j = i+1 to tempstr
         call setprofile(app_hini, group_name, 'FILE'j, '')
         call setprofile(app_hini, group_name, 'POSN'j, '')
      enddo
   endif
compile if INCLUDE_DESKTOP_SUPPORT -- Ask whether to include on Desktop?
 compile if EVERSION < 6  -- Don't need to check if EPM 6.00!
   if dos_version() >= 2000 then  -- 1.3 doesn't have desktop
 compile endif
   if MBID_YES = winmessagebox('Save Group', 'Add a program object to the OS/2 desktop for this group?', 16404) then  -- MB_YESNO + MB_ICONQUESTION + MB_MOVEABLE
 compile if EVERSION>=6
      tib_ptr = 1234                /* 4-byte place to put a far pointer */
      pib_ptr = 1234
      call dynalink32('DOSCALLS',           /* dynamic link library name   */
                    '#312',               /* ordinal value for DOS32GETINFOBLOCKS */
                    address(tib_ptr) ||
                    address(pib_ptr) )
      sayerror 'tib_ptr =' c2x(tib_ptr) 'pib_ptr =' c2x(pib_ptr)
      pib = peek(itoa(rightstr(pib_ptr,2),10), itoa(leftstr(pib_ptr,2),10), 28)
      epm_cmd = peekz(substr(pib, 13, 4))

      class_name = "WPProgram"\0
                      /* ^ = ASCII 94 = 'hat' */
      title = "EPM Group:^"group_name\0
      setup_string = "EXENAME="epm_cmd";PROGTYPE=PM;STARTUPDIR="directory()";PARAMETERS='loadgroup" group_name"';"\0
      location = "<WP_DESKTOP>"\0
      rc = 0
      hobj=dynalink32('PMWP',           /* dynamic link library name   */
                      '#281',           -- 'WinCreateObject'
                      address(class_name)   ||
                      address(title)        ||
                      address(setup_string) ||
                      address(location)     ||
                      atol(CO_REPLACEIFEXISTS), 2)
;     if rc then hobj = hobj'; rc='rc '-' sayerrortext(rc); endif
;     sayerror 'hobject =' hobj
      if not hobj then
         sayerror 'Unable to create the program object in the Desktop folder'
      endif
 compile else  -- Can't do it directly; call a command file:
      seg_ptr = 1234                /* 4-byte place to put a far pointer */
      cmd_ptr = 1234
      call dynalink('DOSCALLS',           /* dynamic link library name   */
                    '#91',                /* ordinal value for DOSGETENV */
                    selector(seg_ptr) ||  /* pointer to env. segment     */
                    offset(seg_ptr)   ||
                    selector(cmd_ptr) ||  /* pointer to offset after     */
                    offset(cmd_ptr)     ) /*    COMSPEC; we don't need   */
      env_seg=itoa(seg_ptr,10)
      cmd_ofs=ltoa(cmd_ptr,10)
      epm_cmd=peekz(env_seg, cmd_ofs)
      'rx makegrp' epm_cmd'*'group_name  -- Requires separate MakeGrp.erx
 compile endif
   endif
 compile if EVERSION < 6
   endif  -- dos_version()
 compile endif
compile endif  -- INCLUDE_DESKTOP_SUPPORT

defc loadgroup =
   universal app_hini
   getfileid startfid
   group_name = arg(1)
   if group_name='' then
      group_name = entrybox('Group name')
   endif
   if group_name='' then
      return
   endif
   howmany = queryprofile( app_hini,  group_name, 'ENTRIES')
   if howmany='' then
      sayerror 'Group unknown.'
      return
   endif
   do i=1 to howmany
 compile if EVERSION >= '5.21'
      display -8
 compile endif
      sayerror 'Loading file' i 'of' howmany
 compile if EVERSION >= '5.21'
      display 8
 compile endif
      this_file = queryprofile(app_hini, group_name, 'FILE'i)
      if leftstr(this_file, 5)='.DOS ' then
         subword(this_file, 2)  -- execute the command
      elseif this_file=UNNAMED_FILE_NAME then
         'xcom e /n'
      else
         'e' this_file
      endif
      if not rc | rc=sayerror('Lines truncated') then
         call prestore_pos(queryprofile(app_hini, group_name, 'POSN'i))
      endif
   enddo
   activatefile startfid
   nextfile

/*------------ Cut here --------------------------------------

/***********************************************************/
/* MakeGrp.erx, for use by EPM's SaveFldr command.         */
/***********************************************************/

parse arg epm_cmd'*'group_name
if group_name='' then
   parse source . . me .
   'sayerror 'me':  Missing group name.'
   exit 4
endif

If RxFuncQuery('SysLoadFuncs') Then Do
  Call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
  Call SysLoadFuncs
End
                                          /* ^ = ASCII 94 = 'hat' */
Call SysCreateObject "WPProgram", "EPM Group:^"group_name, "<WP_DESKTOP>",,
    "EXENAME="epm_cmd";PROGTYPE=PM;STARTUPDIR="directory() || ,
    ";PARAMETERS='loadgroup" group_name"';", "U"
If Result <> 1 Then "Sayerror Unable to create the program object in the Desktop folder"

Exit 0

-----------------------  Cut here --------------------------*/
